/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packets.h
 * Datagram definitions for libpathport
 * Copyright (C) 2006 Simon Newton
 */

#ifndef _defined_pathport_packets_h
#define _defined_pathport_packets_h

#include <sys/types.h>
#include <stdint.h>
#include <netinet/in.h>
#include <pathport/common.h>

/*
 * Pathport opcodes
 */
enum pathport_packet_type_e {
  PATHPORT_DATA = 0x0100,
  PATHPORT_PATCH = 0x0200,
  PATHPORT_PATCHREP = 0x0210,
  PATHPORT_GET = 0x0222,
  PATHPORT_GETREP = 0x0223,
  PATHPORT_ARPREQ = 0x0301,
  PATHPORT_ARPREP = 0x0302,
  PATHPORT_SET = 0x0400,
}__attribute__( ( packed ) );

typedef enum pathport_packet_type_e pathport_packet_type_t;


/*
 * Pathport xDmx
 */
struct pathport_pdu_data_s {
  uint16_t type;
  uint16_t num;
  uint8_t res;
  uint8_t sc;
  uint16_t offset;
  uint8_t data[0];
}__attribute__( ( packed ) );

typedef struct pathport_pdu_data_s pathport_pdu_data;


/*
 * Pathport get request
 */
struct pathport_pdu_get_s {
  uint16_t params[0];
} __attribute__( ( packed ) );

typedef struct pathport_pdu_get_s pathport_pdu_get;


/*
 * Pathport get reply
 */
struct pathport_pdu_getrep_s {
  uint8_t params[0];
} __attribute__( ( packed ) );

typedef struct pathport_pdu_getrep_s pathport_pdu_getrep;


struct pathport_pdu_getrep_alv_s {
  uint16_t type;
  uint16_t len;
  uint8_t val[0];
};
typedef struct pathport_pdu_getrep_alv_s pathport_pdu_getrep_alv;

/*
 * Pathport arp
 */
struct pathport_pdu_arpreq_s {}__attribute__( ( packed ) );
typedef struct pathport_pdu_arpreq_s pathport_pdu_arpreq;

/*
 * Pathport arp reply
 */
struct pathport_pdu_arprep_s {
  uint32_t id;
  uint32_t ip;
  uint8_t mcode;  // manufacturer code
  uint8_t dclass;  // device class
  uint8_t dtype;  // device type
  uint8_t num_co;  // number of dmx components
}__attribute__( ( packed ) );

typedef struct pathport_pdu_arprep_s pathport_pdu_arprep;


struct pathport_pdu_header_s {
  uint16_t type;    // pdu type
  uint16_t len;    // length
}__attribute__( ( packed ) );

typedef struct pathport_pdu_header_s pathport_pdu_header;

/*
 * PDU Header
 */
struct pathport_packet_pdu_s {
  pathport_pdu_header head;
  union {
    pathport_pdu_data data;
    pathport_pdu_get get;
    pathport_pdu_getrep getrep;
    pathport_pdu_arpreq arpreq;
    pathport_pdu_arprep arprep;
  } d;    // pdu data
}__attribute__( ( packed ) );

typedef struct pathport_packet_pdu_s pathport_packet_pdu;


/*
 * A complete Pathport packet
 */
struct pathport_packet_header_s {
  uint16_t proto;    // protocol
  uint16_t ver;    // version
  uint16_t seq;      // seq numb
  uint8_t res[6];    // set to 0
  uint32_t src;    // src id
  uint32_t dst;    // dst id
}__attribute__( ( packed ) );

typedef struct pathport_packet_header_s pathport_packet_header;


/*
 * the complete pathport packet
 */
struct pathport_packet_s {
  pathport_packet_header head;
  uint8_t data[1480];    // pad to 1500 bytes
};

typedef struct pathport_packet_s pathport_packet;


/*
 * Our representation of a  packet, containing the acutal packet
 * plus the length and a src/dst address
 */
typedef struct {
  int length;
  struct in_addr from;
  struct in_addr to;
  pathport_packet p;
} pathport_ipacket_t;

typedef pathport_ipacket_t *pathport_ipacket;

#endif
